/* License of this file: CC0 Public Domain Dedication along
   see http://creativecommons.org/publicdomain/zero/1.0/ */

#include <stdio.h>
#include <stdlib.h>

#include <libmbtk/libmbtk.h>
#include <libmbtk/backend/sdl2.h>
#include <libmbtk/widget.h>
#include <libmbtk/style_f3d.h>

static char *font[] = {"../../../font/Vera.ttf,size=15"};
static mbtk_backend_t be;
static mbtk_display_t disp;

/* Initialize be and disp and set up f3d for default style. Exits on error. */
static void example_init_sdl2(int *argc, char **argv[])
{
	if (mbtk_sdl2_backend(&be) != 0) {
		fprintf(stderr, "Unable to set up the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_be_init(&be) != 0) {
		fprintf(stderr, "Unable to initialize the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_display_init(&be, &disp, argc, argv) != 0) {
		fprintf(stderr, "Unable to initialize mbtk display with the sdl2 backend\n");
		exit(1);
	}

	/* Set default style and font ID for the display */
	mbtk_f3d_style(&disp);
	disp.default_fid = mbtk_resolve_font(&disp, 1, font);
}


/* Hide (or show) right side widget if the associated checkbox is checked or
   unchecked. */
static mbtk_event_handled_t chk_cb(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	mbtk_widget_t *right_side = user_data;

	if (w->flags.activated)
		mbtk_widget_hide(right_side, 1);
	else
		mbtk_widget_show(right_side, 1);
	return 0;
}

/* Create and return a hbox with a checkbox and a label */
static mbtk_box_t *create_row(int id)
{
	mbtk_box_t *row;
	mbtk_checkbox_t *chk;
	mbtk_label_t *lab;
	char tmp[32];


	row = mbtk_hbox_new(NULL);

	/* create checkbox */
	chk = mbtk_checkbox_new(NULL);
	mbtk_box_add_widget(row, &chk->w, 0);

	/* create label */
	sprintf(tmp, "This is\nrow #%d", id);
	lab = mbtk_label_new(NULL, tmp);
	mbtk_box_add_widget(row, &lab->w, 0);

	/* Now that we have the label ready, bind checkbox callback */
	mbtk_checkbox_callback_toggle(chk, chk_cb, lab);

	return row;
}

int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_box_t *root, *row;
	int n;

	example_init_sdl2(&argc, &argv);

	/* create a small, normal (non-modal, non-menu) window. -1 for coord means
	   we do not have a request on placement coords; the last two are recommended
	   initial window size */
	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk hello world", -1, -1, 100, 150);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}

	/* create the root widget as a vbox so child widgets are packed vertically;
	   default alignment is "justify", which means excess space are put between
	   the widgets. */
	root = mbtk_vbox_new(NULL);
	mbtk_window_set_root_widget(win, &root->w);

	/* create 4 rows */
	for(n = 0; n < 4; n++) {
		row = create_row(n);
		mbtk_box_add_widget(root, &row->w, 0);
	}

	/* main loop: run the GUI until win is closed */
	mbtk_run_window(win);

	/* Uninit everyhting at once */
	mbtk_uninit();
}
