/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <stdlib.h>

#define INCLUDE_TIME_SUBSEC
#include "../../config.h"

#include "monotime.h"

#ifdef MBTK_HAVE_CLOCK_MONOTONIC_RAW
#	define MBTK_MONOTONIC_CLOCK CLOCK_MONOTONIC_RAW
#else
#	define MBTK_MONOTONIC_CLOCK CLOCK_MONOTONIC
#endif

double mbtk_be_get_monotime(void)
{
#if defined(MBTK_HAVE_CLOCK_GETTIME)
	struct timespec now;
	clock_gettime(MBTK_MONOTONIC_CLOCK, &now);
	return (double)now.tv_sec + (double)(now.tv_nsec) / 1000000000.0;
#elif defined(MBTK_HAVE_MACH_ABSOLUTE_TIME)
	uint64_t now = mach_absolute_time();
	return ((double)((now * mach_base_info.numer) / mach_base_info.denom) / 1000000000.0);
#elif defined(MBTK_HAVE_TIMEGETTIME)
	return (doubel)timeGetTime() / 1000.0;
#elif defined(MBTK_HAVE_QUERYPERFORMANCECOUNTER)
	static LARGE_INTEGER freq = 0;
	LARGE_INTEGER now;
	if (freq == 0)
		QueryPerformanceFrequency(&freq);
	return (double)now.QuadPart / (double)hires_ticks_per_second.QuadPart;
#elif defined(MBTK_HAVE_GETTIMEOFDAY)
	struct timeval now;
	gettimeofday(&now, NULL);
	return (double)(now.tv_sec) + (double)(now.tv_usec) / 1000000.0;
#elif defined(MBTK_HAVE_FTIME)
	struct timeb tb;
	ftime(&tb);
	return (double)tb.time + (double)(tb.millitm) / 1000.0;
#else
	mbtk_error("mbtk_be_get_monotime(): No time source available on this system.");
	abort();
#endif
	return -1;
}
