/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

static int mbtk_glfw_display_init(mbtk_backend_t *be, mbtk_display_t *disp, int *argc, char **argv[])
{
	glfw_display_t *gs = (glfw_display_t *)disp;
	static uall_sysalloc_t sysa = {
		sizeof(mbtk_glfw_event_t) * 128,
		uall_stdlib_alloc, uall_stdlib_free,
		NULL
	};

	mbtk_be_check_priv_sizes(glfw_display_t, glfw_win_t);

	if (FT_Init_FreeType(&gs->lib) != 0)
		return -1;

	memset(&gs->evs, 0, sizeof(uall_slabs_t));
	gs->evs.sys = &sysa;
	gs->evs.slab_size = sizeof(mbtk_glfw_event_t);
	gs->evq = gs->evq_tail = NULL;

	memset(&gs->iot_fdwatch, 0, sizeof(gs->iot_fdwatch));

	gs->base_time = glfwGetTime();
	return 0;
}

static void mbtk_glfw_watch_fd_uninit(mbtk_display_t *disp);
static void mbtk_glfw_watch_stdin_uninit(mbtk_display_t *disp);

static int mbtk_glfw_display_uninit(mbtk_backend_t *be, mbtk_display_t *disp)
{
	glfw_display_t *gs = (glfw_display_t *)disp;

	uall_slab_clean(&gs->evs);
	gs->evq = gs->evq_tail = NULL;
	FT_Done_FreeType(gs->lib);

	mbtk_glfw_watch_fd_uninit(disp);
	mbtk_glfw_watch_stdin_uninit(disp);

	return 0;
}

static double mbtk_glfw_get_time_el(mbtk_display_t *disp)
{
	glfw_display_t *gd = (glfw_display_t *)disp;
	return glfwGetTime() - gd->base_time;
}

static mbtk_keymod_t mbtk_glfw_query_kbd_mods(mbtk_display_t *disp)
{
	glfw_win_t *gw = (glfw_win_t *)disp->topwin;
	mbtk_keymod_t res = 0;

	if (glfwGetKey(gw->window, GLFW_KEY_LEFT_SHIFT) || glfwGetKey(gw->window, GLFW_KEY_RIGHT_SHIFT))
		res |= MBTK_KM_SHIFT;
	if (glfwGetKey(gw->window, GLFW_KEY_LEFT_ALT) || glfwGetKey(gw->window, GLFW_KEY_RIGHT_ALT))
		res |= MBTK_KM_ALT;
	if (glfwGetKey(gw->window, GLFW_KEY_LEFT_CONTROL) || glfwGetKey(gw->window, GLFW_KEY_RIGHT_CONTROL))
		res |= MBTK_KM_CTRL;

	return res;
}
