/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

static mbtk_window_t *mbtk_sdl2_id2win(Uint32 wid)
{
	SDL_Window *sdlw = SDL_GetWindowFromID(wid);
	if (sdlw == NULL)
		return NULL;
	return SDL_GetWindowData(sdlw, "w");
}

static sdl2_win_t *mbtk_sdl2_get_subw(sdl2_win_t *parent_win, long x, long y, long *subx, long *suby)
{
	sdl2_win_t *subwin;
	for(subwin = gdl_first(&parent_win->subwins); subwin != NULL; subwin = subwin->link.next) {
		mbtk_window_t *sw = (mbtk_window_t *)subwin;
		if ((x >= sw->x) && (y >= sw->y) && (x < sw->x + sw->w) && (y < sw->y + sw->h))
			return mbtk_sdl2_get_subw(subwin, x - sw->x, y - sw->y, subx, suby);
	}
	*subx = x; *suby = y;
	return parent_win;
}

static void mbtk_sdl2_subwin_coords(sdl2_win_t *sw, long *dst_x, long *dst_y, long src_x, long src_y)
{
	*dst_x = src_x;
	*dst_y = src_y;
	if (sw->subwin) {
		mbtk_window_t *win = (mbtk_window_t *)sw;
		(*dst_x) -= win->x;
		(*dst_y) -= win->y;
	}
}

static int mbtk_sdl2_proc_stdin(SDL_Event ev, mbtk_event_t *dst)
{
	mbtk_display_t *disp = ev.user.data1;
/*	sdl2_display_t *sd = ev.user.data1;*/

	dst->type = MBTK_EV_RSTDIN;
	dst->win = disp->topwin;
	dst->data.rstdin.len = iot_stdio.len;
	dst->data.rstdin.data = iot_stdio.data;

/*	printf("STDIO EVENT %d '%s'!\n", iot_stdio.len, iot_stdio.data);*/

	SDL_SemTryWait(iot_stdio.enable); /* decrease to 0 to make sure the therad waits before sending the next event */
	if (!iot_stdio.disabled)
		SDL_SemPost(iot_stdio.enable); /* enable sending the next event */
	SDL_SemPost(iot_stdio.handled); /* release data */
	return 1;
}

static int mbtk_sdl2_proc_fdwatch(SDL_Event ev, mbtk_event_t *dst)
{
	mbtk_display_t *disp = ev.user.data1;
	sdl2_display_t *sd = ev.user.data1;

	dst->type = MBTK_EV_FDWATCH;
	dst->win = disp->topwin;
	dst->data.fdwatch.fd = sd->iot_fdwatch.fd;
	dst->data.fdwatch.readable = sd->iot_fdwatch.readable;
	dst->data.fdwatch.writable = sd->iot_fdwatch.writable;
	dst->data.fdwatch.closed = sd->iot_fdwatch.closed;

/*	printf("FDWATCH event fd=%d\n", dst->data.fdwatch.fd);*/

	SDL_SemPost(sd->iot_fdwatch.handled); /* release data */
	return 1;
}

static int mbtk_sdl2_proc_sdl2ev(SDL_Event ev, mbtk_event_t *dst)
{
	SDL_Event ev2;
	int r;

	switch(ev.type) {
		case SDL_WINDOWEVENT:
			switch(ev.window.event) {
				case SDL_WINDOWEVENT_EXPOSED:
					dst->type = MBTK_EV_WIN_EXPOSE;
					dst->win = mbtk_sdl2_id2win(ev.window.windowID);
					dst->data.win.x = dst->data.win.y = dst->data.win.w = dst->data.win.h = 0;
					return 1;
				case SDL_WINDOWEVENT_MOVED:
					dst->type = MBTK_EV_WIN_MOVE;
					dst->win = mbtk_sdl2_id2win(ev.window.windowID);
					dst->data.win.x = ev.window.data1;
					dst->data.win.y = ev.window.data2;
					dst->data.win.w = dst->data.win.h = 0;
					return 1;
				case SDL_WINDOWEVENT_RESIZED:
					dst->type = MBTK_EV_WIN_RESIZE;
					dst->win = mbtk_sdl2_id2win(ev.window.windowID);
					dst->data.win.x = dst->data.win.y = 0;
					dst->data.win.w = ev.window.data1;
					dst->data.win.h = ev.window.data2;
					return 1;
				case SDL_WINDOWEVENT_CLOSE:
					dst->type = MBTK_EV_WIN_CLOSE;
					dst->win = mbtk_sdl2_id2win(ev.window.windowID);
					dst->data.win.x = dst->data.win.y = dst->data.win.w = dst->data.win.h = 0;
					return 1;
				case SDL_WINDOWEVENT_ENTER:
					dst->type = MBTK_EV_WIN_ENTER;
					dst->win = mbtk_sdl2_id2win(ev.window.windowID);
					dst->data.win.x = dst->data.win.y = dst->data.win.w = dst->data.win.h = 0;
					return 1;
				case SDL_WINDOWEVENT_LEAVE:
					dst->type = MBTK_EV_WIN_LEAVE;
					dst->win = mbtk_sdl2_id2win(ev.window.windowID);
					dst->data.win.x = dst->data.win.y = dst->data.win.w = dst->data.win.h = 0;
					return 1;
			}
			break;

		case SDL_KEYUP:
		case SDL_KEYDOWN:
			dst->type = (ev.type == SDL_KEYDOWN) ? MBTK_EV_KEY_PRESS : MBTK_EV_KEY_RELEASE;
			dst->win = mbtk_sdl2_id2win(ev.key.windowID);
			mbtk_sdl2_key_xlate(dst, &ev);
			dst->data.key.edit = 0;
			/* check if there's a text input event available for the same keyup  and merge */
			if (ev.type == SDL_KEYUP) {
				r = SDL_PeepEvents(&ev2, 1, SDL_PEEKEVENT, SDL_FIRSTEVENT, SDL_LASTEVENT);
				if ((r == 1) && (ev2.type == SDL_TEXTINPUT) && (ev.key.timestamp == ev2.text.timestamp)) {
					SDL_PollEvent(&ev2);
					mbtk_sdl2_text_xlate(dst, &ev2);
				}
			}
			return 1;

		case SDL_TEXTINPUT:
			r = SDL_PeepEvents(&ev2, 1, SDL_PEEKEVENT, SDL_FIRSTEVENT, SDL_LASTEVENT);
			/* check if there's a keyup event available for the same keyup  and merge */
			if ((r == 1) && (ev2.type == SDL_KEYUP) && (ev.key.timestamp == ev2.text.timestamp)) {
				SDL_PollEvent(&ev2);
				dst->type = MBTK_EV_KEY_RELEASE;
				dst->win = mbtk_sdl2_id2win(ev.text.windowID);
				mbtk_sdl2_key_xlate(dst, &ev2);
				dst->data.key.edit = ev.text.text[0];
			}
			else {
				dst->type = MBTK_EV_KEY_RELEASE;
				dst->win = mbtk_sdl2_id2win(ev.text.windowID);
				dst->data.key.scancode = 0;
				dst->data.key.sym = 0;
				dst->data.key.mods = 0;
				mbtk_sdl2_text_xlate(dst, &ev);
			}
			return 1;

		case SDL_MOUSEBUTTONDOWN:
		case SDL_MOUSEBUTTONUP:
			{
				mbtk_window_t *win = mbtk_sdl2_id2win(ev.button.windowID);
				sdl2_win_t *topwin = (sdl2_win_t *)win;
				sdl2_win_t *sw = ((topwin != NULL) && (topwin->last_mouse_subw != NULL)) ? topwin->last_mouse_subw : topwin;
				static mbtk_window_t *last_click_win;

				if (win == NULL)
					win = last_click_win;
				else
					last_click_win = win;

				dst->type = (ev.type == SDL_MOUSEBUTTONDOWN) ? MBTK_EV_MOUSE_PRESS : MBTK_EV_MOUSE_RELEASE;
				dst->win = (mbtk_window_t *)sw;
				if (win->entered || ((sw != NULL) && sw->subwin)) {
					long x, y;
					mbtk_sdl2_subwin_coords(sw, &x, &y, ev.button.x, ev.button.y);
					dst->data.mouse.x = x + ((mbtk_window_t *)sw)->content_x0;
					dst->data.mouse.y = y + ((mbtk_window_t *)sw)->content_y0;
				}
				else
					dst->data.mouse.x = dst->data.mouse.y = -1;
				dst->data.mouse.button = mbtk_sdl2_mouse_button_xlate(ev.button.button);

				mbtk_display_set_mouse_button(win->display, dst->data.mouse.button, (ev.type == SDL_MOUSEBUTTONDOWN) ? 1 : 0);
			}
			return 1;

		case SDL_MOUSEMOTION:
			{
				sdl2_win_t *topwin = (sdl2_win_t *)mbtk_sdl2_id2win(ev.button.windowID);
				long subx, suby;

				topwin->last_mouse_subw = mbtk_sdl2_get_subw(topwin, ev.motion.x, ev.motion.y, &subx, &suby);

				dst->type = MBTK_EV_MOUSE_MOTION;
				dst->win = (mbtk_window_t *)topwin->last_mouse_subw;
				dst->data.mouse.x = subx + dst->win->content_x0;
				dst->data.mouse.y = suby + dst->win->content_y0;
			}
			return 1;

		case SDL_MOUSEWHEEL:
			{
				sdl2_win_t *topwin = (sdl2_win_t *)mbtk_sdl2_id2win(ev.button.windowID);
				sdl2_win_t *sw = topwin->last_mouse_subw != NULL ? topwin->last_mouse_subw : topwin;

				dst->type = MBTK_EV_MOUSE_SCROLL;
				dst->win = (mbtk_window_t *)sw;
				dst->data.mouse.x = ev.wheel.x;
				dst->data.mouse.y = ev.wheel.y;
			}
			return 1;

	}
	return 0;
}


static int mbtk_sdl2_iterate(mbtk_display_t *display, mbtk_event_t *dst, double timeout, int block)
{
	SDL_Event ev;

	do {
		int res;

		if (block || (timeout > 0))
			res = SDL_WaitEventTimeout(&ev, timeout*1000.0);
		else
			res = SDL_PollEvent(&ev);

		if (res) {
			if (ev.type == mbtk_sdl2_ev_rstdio)
				return mbtk_sdl2_proc_stdin(ev, dst);
			else if (ev.type == mbtk_sdl2_ev_fdwatch)
				return mbtk_sdl2_proc_fdwatch(ev, dst);
			else if (mbtk_sdl2_proc_sdl2ev(ev, dst))
				return 1;
		}
	} while(block);

	return 0;
}
