/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#define INCLUDE_SELECT
#include "../../config.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <libmbtk/src_3rd/genht/hash.h>
#include "../../libmbtk.h"

#include "../xlib.h"

#include "mbtk_xlib_priv.h"

#include "wintype.c"
#include "display.c"
#include "input_xlate.c"
#include "io_thread.c"
#include "pixmap.c"
#include "clr_cache.c"
#include "draw.c"
#include "font.c"
#include "cursor.c"
#include "clipboard.c"
#include "iterate.c"
#include "window.c"

int mbtk_xlib_backend(mbtk_backend_t *dst)
{
	memset(dst, 0, sizeof(mbtk_backend_t));

	dst->name = "xlib";

	dst->display_init = mbtk_xlib_display_init;
	dst->display_uninit = mbtk_xlib_display_uninit;
	dst->window_new = mbtk_xlib_window_new;
	dst->subwindow_new = mbtk_xlib_subwindow_new;
	dst->subwindow_assign = mbtk_xlib_subwindow_assign;
	dst->subwindow_unassign = mbtk_xlib_subwindow_unassign;
	dst->subwindow_geometry = mbtk_xlib_subwindow_geometry;
	dst->window_close = mbtk_xlib_window_close;
	dst->win2render_win = mbtk_xlib_win2render_win;
	dst->clipboard_set = mbtk_xlib_clipboard_set;
	dst->clipboard_get = mbtk_xlib_clipboard_get;
	dst->clipboard_free = mbtk_xlib_clipboard_free;
	dst->iterate = mbtk_xlib_iterate;
	dst->get_time_el = mbtk_xlib_get_time_el;

	dst->window_set_title = mbtk_xlib_window_set_title;
	dst->window_raise = mbtk_xlib_window_raise;
	dst->window_hide = mbtk_xlib_window_hide;
	dst->window_show = mbtk_xlib_window_show;
	dst->window_transient4 = mbtk_xlib_window_transient4;
	dst->window_set_min_size = mbtk_xlib_window_set_min_size;
	dst->window_set_size = mbtk_xlib_window_set_size;

	dst->draw_color = mbtk_xlib_draw_color;
	dst->draw_fillrect = mbtk_xlib_draw_fillrect;
	dst->draw_line = mbtk_xlib_draw_line;
	dst->draw_hline = mbtk_xlib_draw_hline;
	dst->draw_vline = mbtk_xlib_draw_vline;
	dst->draw_flush = mbtk_xlib_draw_flush;
	dst->draw_start = mbtk_xlib_draw_start;
	dst->draw_set_font = mbtk_xlib_draw_set_font;
	dst->draw_query_text = mbtk_xlib_draw_query_text;
	dst->draw_query_text_pos = mbtk_xlib_draw_query_text_pos;
	dst->draw_text = mbtk_xlib_draw_text;

	dst->resolve_font = mbtk_xlib_resolve_font;

	dst->draw_import_pixmap = mbtk_xlib_draw_import_pixmap;
	dst->draw_free_pixmap = mbtk_xlib_draw_free_pixmap;
	dst->draw_bind_pixmap = mbtk_xlib_draw_bind_pixmap;
	dst->draw_pixmap = mbtk_xlib_draw_pixmap;

	dst->mocu_create_named = mbtk_xlib_mocu_create_named;
	dst->mocu_create_pixmap = mbtk_xlib_mocu_create_pixmap;
	dst->mocu_free_cursor = mbtk_xlib_mocu_free_cursor;
	dst->mocu_set_cursor = mbtk_xlib_mocu_set_cursor;

	dst->query_kbd_mods = mbtk_xlib_query_kbd_mods;

	dst->watch_stdin_read = mbtk_xlib_watch_stdin_read;
	dst->watch_fd = mbtk_xlib_watch_fd;

	dst->has_alpha = 0;

	return 0;
}
