/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <libmbtk/config.h>
#include <stdlib.h>
#include <genht/hash.h>
#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include "callback.h"

mbtk_callback_t mbtk_htcb_invalid = {0};

#define HT_INVALID_VALUE mbtk_htcb_invalid
#define HT(x) mbtk_htcb_ ## x
#include <genht/ht.c>

int mbtk_htcb_keyeq(mbtk_htcb_key_t a, mbtk_htcb_key_t b)
{
	return a == b;
}

unsigned mbtk_htcb_keyhash(mbtk_htcb_key_t k)
{
	return longhash(k);
}

#undef HT

void mbtk_callback_set_noarg(mbtk_widget_t *w, mbtk_kw_t id, mbtk_event_handled_t (*cb)(mbtk_widget_t *w, mbtk_kw_t id, void *user_data), void *user_data)
{
	mbtk_callback_t tmp;

	if (w->callbacks == NULL)
		w->callbacks = mbtk_htcb_alloc(mbtk_htcb_keyhash, mbtk_htcb_keyeq);

	tmp.func.noarg = cb;
	tmp.user_data = user_data;
	tmp.inhibit = 0;
	mbtk_htcb_set(w->callbacks, id, tmp);
}

void mbtk_callback_set_event(mbtk_widget_t *w, mbtk_kw_t id, mbtk_event_handled_t (*cb)(mbtk_widget_t *w, mbtk_kw_t id, mbtk_event_t *ev, void *user_data), void *user_data)
{
	mbtk_callback_t tmp;

	if (w->callbacks == NULL)
		w->callbacks = mbtk_htcb_alloc(mbtk_htcb_keyhash, mbtk_htcb_keyeq);

	tmp.func.event = cb;
	tmp.user_data = user_data;
	tmp.inhibit = 0;
	mbtk_htcb_set(w->callbacks, id, tmp);
}

void mbtk_callback_uninit(mbtk_widget_t *w)
{
	if (w->callbacks == NULL)
		return;
	mbtk_htcb_free(w->callbacks);
	w->callbacks = NULL;
}

