/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_CALLBACK_IMP_H
#define LIBMBTK_CALLBACK_IMP_H

/* API: Callback helpers inteded for widget implementations. */

#include <libmbtk/config.h>
#include <libmbtk/callback.h>

/* Generic callback sets; shouldn't be used directly by the application but
   through widget specific callback set functions,
   e.g. mbtk_button_callback_click()  */
void mbtk_callback_set_noarg(mbtk_widget_t *w, mbtk_kw_t id, mbtk_event_handled_t (*cb)(mbtk_widget_t *w, mbtk_kw_t id, void *user_data), void *user_data);
void mbtk_callback_set_event(mbtk_widget_t *w, mbtk_kw_t id, mbtk_event_handled_t (*cb)(mbtk_widget_t *w, mbtk_kw_t id, mbtk_event_t *ev, void *user_data), void *user_data);

/* Helper functions to perform a callback on a widget. Calls widget's
   callbeck named in id with the arguments passed (user_data passed on
   callback registration is appended automatically to the arg list). Returns
   what the user callback returns; if user callback is not bound, returns
   MBTK_EVENT_NOT_HANDLED. */
MBTK_INLINE mbtk_event_handled_t mbtk_callback_noarg(mbtk_widget_t *widget, mbtk_kw_t id);
MBTK_INLINE mbtk_event_handled_t mbtk_callback_event(mbtk_widget_t *widget, mbtk_kw_t id, mbtk_event_t *ev);

/*** Implementation (doc-off) ***/

MBTK_INLINE mbtk_event_handled_t mbtk_callback_noarg(mbtk_widget_t *widget, mbtk_kw_t id)
{
	mbtk_callback_t cb;

	if (widget->callbacks == NULL) return 0;

	cb = mbtk_htcb_get(widget->callbacks, id);
	if ((cb.func.noarg == NULL) || cb.inhibit) return 0;

	return cb.func.noarg(widget, id, cb.user_data);
}

MBTK_INLINE mbtk_event_handled_t mbtk_callback_event(mbtk_widget_t *widget, mbtk_kw_t id, mbtk_event_t *ev)
{
	mbtk_callback_t cb;

	if (widget->callbacks == NULL) return MBTK_EVENT_NOT_HANDLED;

	cb = mbtk_htcb_get(widget->callbacks, id);
	if ((cb.func.event == NULL) || cb.inhibit) return MBTK_EVENT_NOT_HANDLED;

	return cb.func.event(widget, id, ev, cb.user_data);
}




#endif
