/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_KEYSYM_H
#define LIBMBTK_KEYSYM_H

/* API: key symbols (portable names for keycodes) */

typedef enum mbtk_keymod_e { /* bitfield */
	MBTK_KM_SHIFT = 1,
	MBTK_KM_ALT = 2,
	MBTK_KM_CTRL = 4,
	MBTK_KM_KEYPAD = 8    /* pressed on the keymap */
} mbtk_keymod_t;

typedef enum mbtk_keysym_e {
	MBTK_KS_INVALID = 0,  /* failed to decode */

	/* values between 32 and 127 are ASCII, not defined here */

	/* values below 32 are invalid, except for these: */
	MBTK_KS_BACKSPACE = 8,
	MBTK_KS_TAB = 9,
	MBTK_KS_ENTER = 10, /* for both \r and \n */
	MBTK_KS_ESC = 27,

	/* F1..Fx: use MBTK_KS_FN + x where x <= 255; so F1 is MBTK_KS_FN+1 (F0 is not used) */
	MBTK_KS_FN = 0xF000,

	/* Cursor */
	MBTK_KS_HOME = 0xF100,   /* also begin */
	MBTK_KS_END,
	MBTK_KS_LEFT,
	MBTK_KS_RIGHT,
	MBTK_KS_UP,
	MBTK_KS_DOWN,
	MBTK_KS_PGUP,           /* also prior */
	MBTK_KS_PGDN,           /* also next */

	/* Modifiers */
	MBTK_KS_SHIFT_L = 0xF200,
	MBTK_KS_SHIFT_R,
	MBTK_KS_ALT_L,
	MBTK_KS_ALT_R,
	MBTK_KS_CTRL_L,
	MBTK_KS_CTRL_R,

	/* Random extra keys */
	MBTK_KS_DEL = 0xFF00,
	MBTK_KS_INS
} mbtk_keysym_t;

/* Evaluates to 1 if mbtk_keysym_t ks is a modifier key */
#define mbtk_ks_is_modifier(ks) (((ks) & 0xFF00) == 0xF200)

/* Convert between keysym and human readable key name */
const char *mbtk_keysym2name(mbtk_keysym_t ks);
mbtk_keysym_t mbtk_name2keysym(const char *name);

/*** Internal ***/
void mbtk_keysym_uninit(void);

#endif
