/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_KEYWORDS_H
#define LIBMBTK_KEYWORDS_H
#include <libmbtk/global_typedefs.h>

/* API: string->integer keywords; see doc/ref/keywords.html */


/* Convert a name to a keyword; allocate new keyword value if needed */
mbtk_kw_t mbtk_kw(const char *name);

/* Return a new unique keyword that has no name or string representation; useful
   when a handle is needed e.g. for a pixmap registration and the handle will
   be remembered by mbtk_kw_t but never resolved by string. */
mbtk_kw_t mbtk_kw_uniq(void);

/*** Internal ***/

/* Free all keywords */
void mbtk_kw_uninit(void);

#define mbtk_kw_cache(dst, name) \
do { \
	static mbtk_kw_t cache = mbtk_kw_invalid; \
	if (cache == mbtk_kw_invalid) \
		cache = mbtk_kw(name); \
	dst = cache; \
} while(0)

#endif
