/*  genhvbox - generic hbox/vbox/table widget layout calculator
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/genhvbox
      contact lead developer: http://www.repo.hu/projects/genhvbox/contact.html
*/

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include "hvbox.h"

static void dump_span(FILE *f, hvbox_span_t span)
{
	if (span == HVBOX_MINIMAL)
		fprintf(f, "min");
	else if (span == HVBOX_FILL)
		fprintf(f, "fill");
	else
		fprintf(f, "%d%%", span);
}

static void dump_align(FILE *f, hvbox_align_t a, char *c)
{
	if ((a >= 0) && (a <= HVBOX_RIGHT_BOTTOM))
		fprintf(f, "%c", c[a]);
	else
		fprintf(f, "?");
}

void hvbox_dump_node(FILE *f, int level, hvbox_t *node, int widget_is_str)
{
	char ind[128];
	char *stype;

	assert(level < sizeof(ind)-1);
	if (level > 0)
		memset(ind, ' ', level);
	ind[level] = '\0';

	switch(node->type) {
		case HVBOX_ROOT: stype = "root"; break;
		case HVBOX_HBOX: stype = "hbox"; break;
		case HVBOX_VBOX: stype = "vbox"; break;
		case HVBOX_TABLE: stype = "table"; break;
		case HVBOX_WIDGET: stype = "widget"; break;
		default: stype = "???";
	}

	fprintf(f, "%s%s min=%dx%d (%dx%d) %s=%dx%d at %d;%d", ind, stype, node->w_min, node->h_min, node->w_min_calc, node->h_min_calc, (node->dirty ? "CALC" : "calc"), node->w, node->h, node->x, node->y);

	fprintf(f, " span=");
	dump_span(f, node->span);

	if (node->type != HVBOX_TABLE) {
		fprintf(f, " align=");
		if (node->type == HVBOX_HBOX)
			dump_align(f, node->align, "#.<>");
		else if (node->type == HVBOX_VBOX)
			dump_align(f, node->align, "#.^v");
		else
			fprintf(f, "n/a");
	}
	else
		fprintf(f, " tbl_cols=%d hom=c%d;r%d", node->tbl_cols, node->tbl_hom_cols, node->tbl_hom_rows);

	if (node->hidden)
		fprintf(f, "H");
	else if (node->hidden_calc)
		fprintf(f, "h");
	else
		fprintf(f, " ");

	if (widget_is_str)
		fprintf(f, " w='%s'", (char *)node->widget);
	else
		fprintf(f, " w=%p", node->widget);

	fprintf(f, "\n");
}

void hvbox_dump_nodes(FILE *f, int level, hvbox_t *node, int widget_is_str)
{
	hvbox_t *n;

	hvbox_dump_node(f, level, node, widget_is_str);
	for(n = node->ch_first; n != NULL; n = n->next)
		hvbox_dump_nodes(f, level+1, n, widget_is_str);
}

#if 0
/* calculated */
	int w_avail, h_avail;      /* available space left after packing up all children */
#endif
