/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_UTIL_H
#define LIBMBTK_UTIL_H

/* API: generic utility/helper/portability functions */

#include <libmbtk/config.h>

/* c89-portable version of strdup() */
char *mbtk_strdup(const char *s);

/* Report an error to stderr, similar to fprintf(stderr, fmt, ...);
   can be used to set a breakpoint in debugger */
void mbtk_error(const char *fmt, ...);

/* Report a warning to stderr, similar to fprintf(stderr, fmt, ...);
   can be used to set a breakpoint in debugger */
void mbtk_warning(const char *fmt, ...);


#define MBTK_MAX(a, b) (((a) > (b)) ? (a) : (b))
#define MBTK_MIN(a, b) (((a) < (b)) ? (a) : (b))

/* mbtk_snprintf() is a portable wrapper around snprintf with a limitation:
   the caller needs to ensure fully rendered string will never be longer
   than 1k (independently of mbtk_snprintf()'s size argument) */
#ifdef MBTK_HAVE_SNPRINTF
#define mbtk_snprintf snprintf
#else
#define mbtk_snprintf mbtk_snprintf_
#endif

#endif
