/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/callback_imp.h>
#include <libmbtk/widget/arrow.h>

#define ARROW_SIZE 10

static int mbtk_arrow_fallback_minsize(mbtk_widget_t *w, long *mx, long *my)
{
	*mx = ARROW_SIZE+2; *my = ARROW_SIZE+2; /* always leave 1 pixel "padding" around */
	return 0;
}

static void mbtk_arrow_fallback_draw(mbtk_widget_t *w)
{
	mbtk_arrow_t *arrow = (mbtk_arrow_t *)w;
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	int ox = w->hvbox.x + (w->hvbox.w - ARROW_SIZE) / 2;
	int oy = w->hvbox.y + (w->hvbox.h - ARROW_SIZE) / 2;
	int n;

	style->set_fg_color(win, sp, w->flags);

	switch(arrow->dir) {
		case MBTK_ARROW_RIGHT:
			for(n = 0; n < ARROW_SIZE; n++)
				mbtk_draw_line(win, ox+n+1, oy+n/2+1-1, ox+n+1, oy+ARROW_SIZE-n/2-1-1);
			break;
		case MBTK_ARROW_LEFT:
			for(n = 0; n < ARROW_SIZE; n++)
				mbtk_draw_line(win, ox+ARROW_SIZE-n-1, oy+n/2+1-2, ox+ARROW_SIZE-n-1, oy+ARROW_SIZE-n/2-1-2);
			break;
		case MBTK_ARROW_DOWN:
			for(n = 0; n < ARROW_SIZE; n++)
				mbtk_draw_line(win, ox+n/2+1, oy+n-1, ox+ARROW_SIZE-n/2-1, oy+n-1);
			break;
		case MBTK_ARROW_UP:
			for(n = 0; n < ARROW_SIZE; n++)
				mbtk_draw_line(win, ox+n/2+1, oy+ARROW_SIZE-n-2, ox+ARROW_SIZE-n/2-1, oy+ARROW_SIZE-n-2);
			break;
	}
}

static void mbtk_arrow_fallback_draw_overlay(mbtk_widget_t *w)
{
}



static const mbtk_widget_imp_t mbtk_arrow_imp = {
	NULL, /* mbtk_arrow_destroy */

	mbtk_arrow_fallback_minsize,
	mbtk_arrow_fallback_draw,
	mbtk_arrow_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	NULL, /* post_assign */
	NULL, /* pre_unassign */

	NULL, /* pre_appear */
	NULL, /* post_disappear */

	NULL  /* pre_child_unlink */
};

mbtk_arrow_t *mbtk_arrow_new(mbtk_arrow_t *caller_alloced, mbtk_arrow_dir_t dir)
{
	mbtk_arrow_t *arrow;
	mbtk_kw_t kw_arrow;

	mbtk_kw_cache(kw_arrow, "arrow");

	mbtk_create_wt(arrow, kw_arrow, &mbtk_arrow_imp, caller_alloced);
	arrow->dir = dir;

	return arrow;
}



