/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/callback_imp.h>
#include <libmbtk/widget/pane_sep.h>
#include <libmbtk/util.h>

static int mbtk_pane_sep_fallback_minsize(mbtk_widget_t *w, long *mx, long *my)
{
	mbtk_pane_sep_t *pane_sep = (mbtk_pane_sep_t *)w;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;
	long l, r, t, b, sz;


	flags.activated = 1;

	style->min_box_frame_size(w->win, sp, flags, &l, &r, &t, &b);
	w->hvbox.fr_l = t; w->hvbox.fr_r = b;
	w->hvbox.fr_t = t; w->hvbox.fr_b = b;
	*mx = (l+r)*2; *my = (t+b)*2;

	sz = MBTK_MAX(l+r, t+b);

	if (pane_sep->horiz)
		*mx += sz * 7;
	else
		*my += sz * 7;

	return 0;
}

static void mbtk_pane_sep_fallback_draw(mbtk_widget_t *w)
{
	mbtk_pane_sep_t *pane_sep = (mbtk_pane_sep_t *)w;
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;
	long l, r, t, b, wi, he, vs, hs, sz;
	int n;

	flags.activated = 1;
	style->min_box_frame_size(win, sp, flags, &l, &r, &t, &b);

	hs = l+r;
	vs = t+b;
	wi = pane_sep->horiz ? w->hvbox.w : vs;
	he = pane_sep->horiz ? hs : w->hvbox.h;

	style->draw_box_frame(win, sp, flags, w->hvbox.x, w->hvbox.y, wi, he);

	sz = MBTK_MAX(hs, vs);
	for(n = -1; n <= 1; n++) {
		long x = w->hvbox.x + wi / 2, y = w->hvbox.y + he / 2;
		if (pane_sep->horiz)
			x += n * sz * 3;
		else
			y += n * sz * 3;

		style->draw_bg_rect(win, sp, flags, x - sz, y - sz, sz*2+1, sz*2+1);
		style->draw_box_frame(win, sp, flags, x - sz, y - sz, sz*2+1, sz*2+1);
	}

}

static void mbtk_pane_sep_fallback_draw_overlay(mbtk_widget_t *w)
{
}



static const mbtk_widget_imp_t mbtk_pane_sep_imp = {
	NULL, /* mbtk_pane_sep_destroy */

	mbtk_pane_sep_fallback_minsize,
	mbtk_pane_sep_fallback_draw,
	mbtk_pane_sep_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	NULL, /* post_assign */
	NULL, /* pre_unassign */

	NULL, /* pre_appear */
	NULL, /* post_disappear */

	NULL  /* pre_child_unlink */
};

mbtk_pane_sep_t *mbtk_pane_sep_new(mbtk_pane_sep_t *caller_alloced, int horiz)
{
	mbtk_pane_sep_t *pane_sep;
	mbtk_kw_t kw_hsep, kw_vsep;

	mbtk_kw_cache(kw_hsep, "pane_hsep");
	mbtk_kw_cache(kw_vsep, "pane_vsep");

	mbtk_create_wt(pane_sep, (horiz ? kw_hsep : kw_vsep), &mbtk_pane_sep_imp, caller_alloced);
	pane_sep->w.hvbox.type = (horiz ? HVBOX_HBOX : HVBOX_VBOX);
	pane_sep->horiz = horiz;

	return pane_sep;
}


