/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_WIDGET_PROGBAR_H
#define LIBMBTK_WIDGET_PROGBAR_H

#include <libmbtk/config.h>
#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>

/* Button: can be clicked a mouse click release or enter release */
typedef struct mbtk_progbar_s {
	mbtk_widget_t w;
	double min, max;
	double curr;        /* interpreted only between min and max (value copied and clamped) */
} mbtk_progbar_t;

/* Create a new progress bar with min/max set to 0/1, curr set to 0. */
mbtk_progbar_t *mbtk_progbar_new(mbtk_progbar_t *caller_alloced);

/* Set curr value of prg. If curr is between min and max, inclusive, it is
set. If curr is out of range and clamp is true, it is clamped and set. Else
error is returned. Returns 0 on success, -1 on error. */
int mbtk_progbar_set_curr(mbtk_progbar_t *prg, double curr, int clamp);

MBTK_WIDGET_INLINES(progbar)

#endif
