/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_WIDGET_SCROLLBAR_H
#define LIBMBTK_WIDGET_SCROLLBAR_H

#include <libmbtk/config.h>
#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>

/* A horizontal or vertical bar for the interactive representation of a range */
typedef struct mbtk_scrollbar_s {
	mbtk_widget_t w;
	double min, max;
	double curr;          /* interpreted only between min and max (value copied and clamped) */
	double len;           /* length of the curr window */
	int thickness;        /* in pixels in the direction perpendicular to the main direction (see ->horiz) */
	unsigned int horiz:1; /* set to 1 for horizontal layout, 0 for vertical */
} mbtk_scrollbar_t;

/* Create a new scrollbar with min/max set to 0/1, curr set to 0. */
mbtk_scrollbar_t *mbtk_scrollbar_new(mbtk_scrollbar_t *caller_alloced, int horiz);

/* Set curr value of prg. If curr is between min and max, inclusive, it is
set. If curr is out of range and clamp is true, it is clamped and set. Else
error is returned. Returns 0 on success, -1 on error. */
int mbtk_scrollbar_set_curr(mbtk_scrollbar_t *prg, double curr, double len, int clamp);

/* Bind the range.moved callback, which is called whenever the user (attempted)
   to change scrollbar curr position */
void mbtk_scrollbar_callback_range_moved(mbtk_scrollbar_t *b, mbtk_event_handled_t (*cb)(mbtk_widget_t *w, mbtk_kw_t id, void *user_data), void *user_data);

MBTK_WIDGET_INLINES(scrollbar)

#endif
