/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <math.h>
#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/draw.h>
#include <libmbtk/util.h>
#include <libmbtk/widget/scrolled_wb.h>
#include <libmbtk/widget/box.h>
#include <libmbtk/widget/Common.h>


static void mbtk_scrolled_wb_imp_destroy(mbtk_widget_t *w) { }

void mbtk_comm_scrolled_wb_update(mbtk_comm_scrolled_wb_t *scwb)
{
	mbtk_kw_t kw_range_val;
	mbtk_arg_t a[4];
	mbtk_window_t *subw = scwb->main.subwin;
	mbtk_widget_t *root;
	

	if (subw == NULL)
		return;

	root = mbtk_window_get_root(subw);
	if (root == NULL)
		return;

	mbtk_kw_cache(kw_range_val, "range.val");

	if ((subw->content_x0 != scwb->last.x0) || (subw->w != scwb->last.win_w) || (root->hvbox.w != scwb->last.cnt_w)) {
		if (scwb->scb_horiz.w.hvbox.parent != NULL) {
			mbtk_scrolled_get_range(&scwb->main, a, 1);
			mbtk_widget_set_propa(&scwb->scb_horiz.w, kw_range_val, a, 4);
		}
		scwb->last.x0 = subw->content_x0;
		scwb->last.win_w = subw->w;
		scwb->last.cnt_w = root->hvbox.w;
	}

	if ((subw->content_y0 != scwb->last.y0) || (subw->h != scwb->last.win_h) || (root->hvbox.h != scwb->last.cnt_h)) {
		if (scwb->scb_vert.w.hvbox.parent != NULL) {
			mbtk_scrolled_get_range(&scwb->main, a, 0);
			mbtk_widget_set_propa(&scwb->scb_vert.w, kw_range_val, a, 4);
		}
		scwb->last.y0 = subw->content_y0;
		scwb->last.win_h = subw->h;
		scwb->last.cnt_h = root->hvbox.h;
	}
}

static void mbtk_scrolled_wb_fallback_draw(mbtk_widget_t *w)
{
	mbtk_scrolled_wb_t *scrolled_wb = (mbtk_scrolled_wb_t *)w;

	mbtk_comm_scrolled_wb_update(&scrolled_wb->scwb); /* won't do anything if it doesn't need to */

	mbtk_comm_frame_fallback_draw(w);
}

static void mbtk_scrolled_wb_pre_child_unlink(mbtk_widget_t *w, mbtk_widget_t *child)
{
	mbtk_scrolled_wb_t *scrolled_wb = (mbtk_scrolled_wb_t *)w;
	if (!w->destroying) {
		if ((child == &scrolled_wb->scwb.scb_vert.w) || (child == &scrolled_wb->scwb.scb_horiz.w) || (child == &scrolled_wb->scwb.main.w)) {
			mbtk_error("mbtk_scrolled_wb: scrollbar or main widget got destroyed (shouldn't happen)\n");
			return;
		}
	}
}

static void mbtk_scroled_wb_scrolled_pre_child_unlink(mbtk_widget_t *w, mbtk_widget_t *child)
{
	mbtk_comm_scrolled_wb_t *scwb = (mbtk_comm_scrolled_wb_t *)w->handler_data;

	if (!scwb->main.w.destroying) {
		mbtk_arg_t a[4] = {0};
		mbtk_kw_t kw_range_val;

		mbtk_kw_cache(kw_range_val, "range.val");
		a[0].type = a[1].type = a[2].type = a[3].type = MBTK_DOUBLE;

		if (scwb->scb_vert.w.hvbox.parent != NULL)
			mbtk_widget_set_propa(&scwb->scb_vert.w, kw_range_val, a, 4);

		if (scwb->scb_horiz.w.hvbox.parent != NULL)
			mbtk_widget_set_propa(&scwb->scb_horiz.w, kw_range_val, a, 4);
	}

	/* don't let scrolled destroy it again */
	if ((scwb->main.root != NULL) && (scwb->main.root->destroying))
		scwb->main.root = NULL;
}



static const mbtk_widget_imp_t mbtk_scrolled_wb_imp = {
	mbtk_scrolled_wb_imp_destroy,

	mbtk_comm_frame_fallback_minsize,
	mbtk_scrolled_wb_fallback_draw,
	mbtk_comm_frame_fallback_draw_overlay,

	NULL, /* mbtk_scrolled_wb_get_propa */
	NULL, /* mbtk_scrolled_wb_set_propa */

	NULL, /* post_assign */
	NULL, /* pre_unassign */

	NULL, /* pre_appear */
	NULL, /* post_disappear */

	mbtk_scrolled_wb_pre_child_unlink
};

static void add_widget(mbtk_widget_t *parent, mbtk_widget_t *w)
{
	mbtk_widget_embedded(parent, w);
	mbtk_hvbox_install(w, parent, NULL, 0);
}

static mbtk_event_handled_t scrolled_cb(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	mbtk_comm_scrolled_wb_t *scwb = (mbtk_comm_scrolled_wb_t *)user_data;
	mbtk_window_t *subw = scwb->main.subwin;
	mbtk_arg_t ha[4], va[4];
	long h, v;
	mbtk_kw_t kw_range_val;

	mbtk_kw_cache(kw_range_val, "range.val");

	if (scwb->scb_horiz.w.hvbox.parent != NULL) {
		if (mbtk_widget_get_propa(&scwb->scb_horiz.w, kw_range_val, ha, 4) != 4) {
			mbtk_error("scrolled_wb: failed to query horizonal scrollbar");
			return MBTK_EVENT_NOT_HANDLED;
		}
		h = floor(ha[2].v.d);
	}
	else
		h = subw->content_x0;

	if (scwb->scb_vert.w.hvbox.parent != NULL) {
		if (mbtk_widget_get_propa(&scwb->scb_vert.w, kw_range_val, va, 4) != 4) {
			mbtk_error("scrolled_wb: failed to query vertical scrollbar");
			return MBTK_EVENT_NOT_HANDLED;
		}
		v = floor(va[2].v.d);
	}
	else
		v = subw->content_y0;

	if ((h != subw->content_x0) || (v != subw->content_y0)) {
		scwb->last.x0 = h;
		scwb->last.y0 = v;
		mbtk_scrolled_set_content_origin(&scwb->main, scwb->last.x0, scwb->last.y0);
	}

	return MBTK_EVENT_HANDLED;
}


static void mbtk_scrolled_wb_update_scrollbar(mbtk_widget_t *parent, mbtk_comm_scrolled_wb_t *scwb, mbtk_scrollbar_t *sb, mbtk_scrolled_wb_vis_t vis, int is_horiz)
{
	int create = 0;

	switch(vis) {
		case MBTK_SCROLLED_WB_NEVER:
			if ((sb->w.hvbox.parent != NULL) && !sb->w.hvbox.hidden)
				mbtk_widget_hide(&sb->w, 1);
			break;

		case MBTK_SCROLLED_WB_ALWAYS:
			if (sb->w.hvbox.parent != NULL) {
				if (sb->w.hvbox.hidden)
					mbtk_widget_show(&sb->w, 1);
			}
			else
				create = 1;
			break;

		case MBTK_SCROLLED_WB_AUTO:
			if (sb->w.hvbox.parent == NULL)
				create = 1; /* always create for auto and then show/hide as needed */
			break;
	}

	if (create) {
		mbtk_scrollbar_new(sb, is_horiz);
		mbtk_scrollbar_callback_range_moved(sb, scrolled_cb, scwb);
		add_widget(parent, &sb->w);
	}
}

void mbtk_comm_scrolled_wb_update_scrollbars(mbtk_widget_t *parent, mbtk_comm_scrolled_wb_t *scwb)
{
	mbtk_scrolled_wb_update_scrollbar(parent, scwb, &scwb->scb_vert, scwb->vis_vert, 0);
	mbtk_scrolled_wb_update_scrollbar(parent, scwb, &scwb->scb_horiz, scwb->vis_horiz,1);
}

static mbtk_widget_imp_t mbtk_scroled_wb_scrolled_imp;
static int mbtk_scroled_wb_scrolled_imp_inited = 0;

void mbtk_comm_scrolled_wb_new(mbtk_widget_t *parent, mbtk_comm_scrolled_wb_t *scwb, mbtk_scrolled_wb_vis_t vis_vert, mbtk_scrolled_wb_vis_t vis_horiz)
{
	parent->hvbox.type = HVBOX_TABLE;
	parent->hvbox.tbl_cols = 2;

	mbtk_scrolled_new(&scwb->main);
	scwb->main.w.hvbox.span = scwb->main.w.hvbox.alt_span = HVBOX_FILL;
	add_widget(parent, &scwb->main.w);

	scwb->vis_horiz = vis_horiz;
	scwb->vis_vert = vis_vert;

	mbtk_comm_scrolled_wb_update_scrollbars(parent, scwb);

	scwb->last.x0 = scwb->last.y0 = -1; /* trigger update on first draw */

	/* take over scrolled implementation so that pre_child_unlink can be handled */
	if (!mbtk_scroled_wb_scrolled_imp_inited) {
		memcpy(&mbtk_scroled_wb_scrolled_imp, scwb->main.w.imp, sizeof(mbtk_widget_imp_t));
		mbtk_scroled_wb_scrolled_imp.pre_child_unlink = mbtk_scroled_wb_scrolled_pre_child_unlink;
		mbtk_scroled_wb_scrolled_imp_inited = 1;
	}
	scwb->main.w.imp = &mbtk_scroled_wb_scrolled_imp;
	scwb->main.w.handler_data = scwb;
}

mbtk_scrolled_wb_t *mbtk_scrolled_wb_new(mbtk_scrolled_wb_t *caller_alloced, mbtk_scrolled_wb_vis_t vis_vert, mbtk_scrolled_wb_vis_t vis_horiz)
{
	mbtk_scrolled_wb_t *scrolled_wb;
	mbtk_kw_t kw_scrolled_wb;

	mbtk_kw_cache(kw_scrolled_wb, "scrolled_wb");

	mbtk_create_wt(scrolled_wb, kw_scrolled_wb, &mbtk_scrolled_wb_imp, caller_alloced);

	mbtk_comm_scrolled_wb_new(&scrolled_wb->w, &scrolled_wb->scwb, vis_vert, vis_horiz);
	return scrolled_wb;
}


int mbtk_scrolled_wb_set_content_origin(mbtk_scrolled_wb_t *scrolled_wb, long cx0, long cy0)
{
	return mbtk_scrolled_set_content_origin(&scrolled_wb->scwb.main, cx0, cy0);
}

void mbtk_scrolled_wb_main_set_minsize(mbtk_scrolled_wb_t *scrolled_wb, long w, long h)
{
	mbtk_scrolled_set_minsize(&scrolled_wb->scwb.main, w, h);
}

void mbtk_scrolled_wb_set_root_widget(mbtk_scrolled_wb_t *scrolled_wb, mbtk_widget_t *widget)
{
	scrolled_wb->scwb.last.x0 = scrolled_wb->scwb.last.y0 = -1; /* trigger update on first draw */
	mbtk_scrolled_set_root_widget(&scrolled_wb->scwb.main, widget);
}

mbtk_widget_t *mbtk_scrolled_wb_subwidget(mbtk_scrolled_wb_t *scrolled_wb, mbtk_kw_t subname)
{
	hvbox_t *ch0 = scrolled_wb->w.hvbox.ch_first;
	mbtk_kw_t kw_scrolled, kw_scv, kw_sch;

	mbtk_kw_cache(kw_scrolled, "scrolled");
	mbtk_kw_cache(kw_scv, "srollbar.vert");
	mbtk_kw_cache(kw_sch, "srollbar.horiz");

	if (subname == kw_scrolled) return (mbtk_widget_t *)ch0;
	if (subname == kw_scv) return (mbtk_widget_t *)ch0->next;
	if (subname == kw_sch) return (mbtk_widget_t *)ch0->next->next;

	return NULL;
}
