/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_WIDGET_SCROLLED_WB_H
#define LIBMBTK_WIDGET_SCROLLED_WB_H

#include <libmbtk/config.h>
#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>
#include <libmbtk/widget/scrolled.h>
#include <libmbtk/widget/scrollbar.h>


/* Composite widget for a table that hosts:
   - a scrollable subwindow which hosts a tree of widgets (this is the main widget)
   - a vertical scrollbar right to the main widget
   - a horizontal scrollbar under the main widget
*/

/* When the scrollbar is visible */
typedef enum mbtk_scrolled_wb_vis_e {
	MBTK_SCROLLED_WB_NEVER,
	MBTK_SCROLLED_WB_ALWAYS,
	MBTK_SCROLLED_WB_AUTO            /* only when needed (content is larger than the subwindow) */
} mbtk_scrolled_wb_vis_t;

/* reusable core in other composite widgets */
typedef struct mbtk_comm_scrolled_wb_s {
	mbtk_scrolled_wb_vis_t vis_vert, vis_horiz; /* user configuration */

	/* subwidgets */
	mbtk_scrolled_t main;
	mbtk_scrollbar_t scb_vert, scb_horiz;

	struct {
		long x0, y0, win_w, win_h, cnt_w, cnt_h;
	} last; /* cached last state so update is done only when needed */
} mbtk_comm_scrolled_wb_t;

typedef struct mbtk_scrolled_wb_s {
	mbtk_widget_t w;
	mbtk_comm_scrolled_wb_t scwb;
} mbtk_scrolled_wb_t;


mbtk_scrolled_wb_t *mbtk_scrolled_wb_new(mbtk_scrolled_wb_t *caller_alloced, mbtk_scrolled_wb_vis_t vis_vert, mbtk_scrolled_wb_vis_t vis_horiz);

void mbtk_scrolled_wb_set_root_widget(mbtk_scrolled_wb_t *scrolled, mbtk_widget_t *widget);

/* Set the minimum size of the main subwidget (the scrolled subwindow);
   this means scrollbars are added as extra width/height, on top of this size.
   For setting a minimum on the final size, call mbtk_scrolled_wb_set_minsize()
   on scrolled_wb instead. */
void mbtk_scrolled_wb_main_set_minsize(mbtk_scrolled_wb_t *scrolled_wb, long w, long h);

/* Set the offset of content: content coords of the first (top-left) pixel shown
   in the subwindow. Clamps both cx0 and cy0 to avoid scrolling beyond content.
   Returns 1 if supplied cx0;cy0 made a change. */
int mbtk_scrolled_wb_set_content_origin(mbtk_scrolled_wb_t *scrolled_wb, long cx0, long cy0);

/* Return sub-widget: scrolled, scrollbar.vert, scrollbar.horiz */
mbtk_widget_t *mbtk_scrolled_wb_subwidget(mbtk_scrolled_wb_t *scrolled_wb, mbtk_kw_t subname);

MBTK_WIDGET_INLINES(scrolled_wb)


/*** Building block for composite widgets ***/
/* Same as mbtk_scrolled_wb_new; scwb is a field in the custom widget struct of
   parent. On genhvbox level parent is converted into a HVBOX_TABLE. All
   contents should be added in (mbtk_scrolled_t *) scwb->main. */
void mbtk_comm_scrolled_wb_new(mbtk_widget_t *parent, mbtk_comm_scrolled_wb_t *scwb, mbtk_scrolled_wb_vis_t vis_vert, mbtk_scrolled_wb_vis_t vis_horiz);

void mbtk_comm_scrolled_wb_update(mbtk_comm_scrolled_wb_t *scwb);

/* Hide/show/create scrollbars - call this after scrollbar visibility changed */
void mbtk_comm_scrolled_wb_update_scrollbars(mbtk_widget_t *parent, mbtk_comm_scrolled_wb_t *scwb);

#endif
