/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_WIDGET_FLAGS_H
#define LIBMBTK_WIDGET_FLAGS_H

/* API: flags that any widget should support */

typedef struct mbtk_widget_flags_s {
	/* widget states supported by all widgets: user accessible */
	unsigned disabled:1;        /* won't react to user input or focus */
	unsigned activated:1;       /* e.g. push button pushed, checkbox ticked */
	unsigned focus:1;
	/* hide is tracked in hvbox->hidden */

	/* widget states supported by all widgets: internal */
	unsigned dirty:1;           /* set to 1 if redraw is required */
	unsigned caller_alloced:1;  /* 1 if caller supplied the storage and mbtk_widget_destroy() won't free it */
	unsigned created:1;         /* 1 if the widget is created, 0 if it is already destroyed */

	unsigned focus_pref:8;      /* preference on the focus - when a new layout is created and the default focus needs to be determined; higher value is more preferred; 0 means the widget doesn't want to get focused */

} mbtk_widget_flags_t;

#endif
