/*  miniboxtk - small GUI toolkit/widget system - test programs
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <libmbtk/libmbtk.h>
#include <libmbtk/widget.h>
#include <libmbtk/style_f3d.h>
#include <libmbtk/arg_t.h>
#include <libmbtk/mpm.h>
#include "../be_init.h"

static char **font;
static int stay;

static mbtk_event_handled_t button_toggle(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	fprintf(stderr, "Toggle: '%s' %d\n", (const char *)user_data, w->flags.activated);
	return 0;
}

int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_backend_t be;
	mbtk_display_t disp;
	mbtk_box_t *vbox, *hbox1;
	mbtk_radiobutton_t *wrb;
	mbtk_handler_hovradio_t *hrr;
	mbtk_textline_t *tln;

	font = tests_backend_init(&be, &disp, &argc, &argv);
	mbtk_f3d_style(&disp);


	disp.default_fid = mbtk_resolve_font(&disp, 1, font);

	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk odd widgets", -1, -1, 300, 300);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}

	vbox = mbtk_vbox_new(NULL);
	mbtk_window_set_root_widget(win, &vbox->w);

	/*** Radio button ***/

	/* create and append a hbox; this will host our example radio buttons */
	hbox1 = mbtk_hbox_new(NULL);
	mbtk_box_add_widget(vbox, &hbox1->w, 0);

	/* create a radio handler within the current widow */
	hrr = mbtk_handler_hovradio_alloc(win);

	/* create a radio button.... */
	wrb = mbtk_radiobutton_new(NULL);
	mbtk_radiobutton_callback_toggle(wrb, button_toggle, "radio 1");
	mbtk_box_add_widget(hbox1, &wrb->w, 0);
	mbtk_handler_hovradio_add_widget(hrr, &wrb->w); /* add to the radio handler; this groups our radio buttons */

	/* create a second radio button.... */
	wrb = mbtk_radiobutton_new(NULL);
	mbtk_radiobutton_callback_toggle(wrb, button_toggle, "radio 2");
	mbtk_box_add_widget(hbox1, &wrb->w, 0);
	mbtk_handler_hovradio_add_widget(hrr, &wrb->w);


	/* create a third button.... */
	wrb = mbtk_radiobutton_new(NULL);
	mbtk_radiobutton_callback_toggle(wrb, button_toggle, "radio 3");
	mbtk_box_add_widget(hbox1, &wrb->w, 0);
	mbtk_radiobutton_style_set_prop(wrb, mbtk_kw("fg.color"), mbtk_arg_str("#ff0000")); /* change the foreground color, which is the color of the mark */
	mbtk_handler_hovradio_add_widget(hrr, &wrb->w);

	/* create a textline.... */
	tln = mbtk_textline_new(NULL, "hello world");
	mbtk_box_add_widget(vbox, &tln->w, 0);

	printf("SET: %d\n", mbtk_textline_cursor_set_chr(tln, 3, -1));


	for(stay = 1; stay;) {
		mbtk_event_t ev;
		if (mbtk_iterate(&disp, &ev, 1)) {
			if (ev.type == MBTK_EV_MOUSE_RELEASE) {
				mbtk_textline_cursor_set_win(tln, ev.data.mouse.x);
			}
			else if (ev.type == MBTK_EV_KEY_RELEASE) {
				if (ev.data.key.sym == 'q')
					break;
				else if (ev.data.key.sym == MBTK_KS_LEFT)
					printf("SET: %d\n", mbtk_textline_cursor_set_chr(tln, tln->cursor-1, tln->overwrite?0:1));
				else if (ev.data.key.sym == MBTK_KS_RIGHT)
					printf("SET: %d\n", mbtk_textline_cursor_set_chr(tln, tln->cursor+1, tln->overwrite?0:1));
				else if (ev.data.key.sym == MBTK_KS_INS) {
					tln->overwrite = !tln->overwrite;
					printf("overwrite: %d\n", tln->overwrite);
					mbtk_widget_needs_redraw(&tln->w);
				}
				else if (ev.data.key.sym == MBTK_KS_HOME)
					mbtk_textline_cursor_home(tln);
				else if (ev.data.key.sym == MBTK_KS_END)
					mbtk_textline_cursor_end(tln);
				else if (ev.data.key.sym == MBTK_KS_DEL)
					mbtk_textline_del(tln, 1);
				else if (ev.data.key.sym == MBTK_KS_BACKSPACE)
					mbtk_textline_backspace(tln);
				else if (isalnum(ev.data.key.sym)) {
					char tmp[2];
					tmp[0] = ev.data.key.sym;
					tmp[1] = '\0';
					mbtk_textline_type_str(tln, tmp);
				}
			}
		}
	}

	mbtk_uninit();
}
